#!/usr/bin/env python

import unittest

from sliders import flextableparser

class TestFlexTableParser(unittest.TestCase):
    
    def test_init_no_args(self):
        parser = flextableparser.FlexTableParser()
        self.assertIsInstance(parser, flextableparser.FlexTableParser)

    def test_init_with_fastqc(self):
        config_file = '../table_schemas/fastqc.json'
        parser = flextableparser.FlexTableParser(config_file)
        self.assertIsInstance(parser, flextableparser.FlexTableParser)

    def test_init_with_flagstat(self):
        config_file = '../table_schemas/flagstat.json'
        parser = flextableparser.FlexTableParser(config_file)
        self.assertIsInstance(parser, flextableparser.FlexTableParser)

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

import sys
import argparse

from . import flextableparser

def parse_args(argv):

    #print(argv)
    parser = argparse.ArgumentParser()
    parser.add_argument(
                        "input_file",
                        type = str,
                        help = 'Input file path')
    parser.add_argument(
                        "-d",
                        "--delimiter",
                        dest = "delimiter",
                        type = str,
                        help = "Use built-in table conversion schema",
                        required = False)

    if len(argv) < 1:
        print("No arguments specified")
        parser.print_help()
        sys.exit()
    args = parser.parse_args(argv)
    return(args)

def main():
    args = parse_args(sys.argv[1:])
    #print(args)
    if args.delimiter:
        flextableparser.transpose(args.input_file, args.delimiter)
    else:
        flextableparser.transpose(args.input_file)

if __name__ == '__main__':
    main()


#!/usr/bin/env python
"""Convert structured text data into table format.

Available functions:
- parse_file: 
"""

__author__ = 'pbilling@stanford.edu (Paul Billing-Ross)'

import re
import pdb
import sys
import json 
import logging 
import datetime

from collections import OrderedDict

def configure_logger(source_type, name=None, file_handle=False):
    # Configure Logger object
    logger = logging.getLogger(source_type)    # Create logger object
    logger.setLevel(logging.DEBUG)

    timestamp = str(datetime.datetime.now()).split()[0]     # yyyy-mm-dd
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")

    # Add logging stream handler
    STREAM = logging.StreamHandler(sys.stderr)
    STREAM.setLevel(logging.DEBUG)
    STREAM.setFormatter(formatter)
    logger.addHandler(STREAM)

    return logger

def transpose(input_file, delimiter=None):
    """Transpose file rows to columns.

    Solution from Ashwini Chaudhary 
    http://stackoverflow.com/questions/10507104/row-to-column-transposition-in-python
    http://stackoverflow.com/users/846892/ashwini-chaudhary
    """

    with open(input_file) as f:
        if delimiter:
            lis = [x.split(delimiter) for x in f]
        else:
            lis = [x.split() for x in f]

    for x in zip(*lis):
        for y in x:
            print(y+'\t', end='')
        print('\n')

class FlexTableParser:
    """Convert structured text data into CSV format.
    """

    def __init__(self, config_file=None):

        self.config = None
        self.columns = None
        self.dimensions = None
        self.overlap = None
        self.match_patterns = []
        self.static_values = OrderedDict()

        self.logger = configure_logger('FlexTableParser')

        if config_file:
            self.configure(config_file)

    def _write_single_row(self, dimension_name, value):
        """Write single row to table.

        Args:
            dimension (str):
            value (int/float): 

        """

        self.logger.info('Writing single row for dimension: {}'.format(dimension_name))
        index = ''  # Fixed at 1 when writing single row
        
        # Combine line-entry values with static values
        table_values = {
                        'dimension': dimension_name,
                        'index': index,
                        'value': value
                       }
        static_values = self.static_values.copy()
        table_values.update(static_values)

        # Populate output string ordered by columns
        strings = [str(table_values[column]) for column in self.columns]
        out_str = ','.join(strings)
        # self.out_fh.write(out_str)
        print(out_str)

    def _write_series_rows(self, dimension, in_fh):
        """Write series of rows.
        """

        dimension_name = dimension['name']
        self.logger.info('Writing series of rows for dimension: {}'.format(dimension_name))
        # Indexes of the delimiter separated line.
        # (I know, I know... need better terminology.)
        index_element = dimension['index']  # type: int
        value_element = dimension['value']  # type: int

        delimiter = dimension['delimiter'] # type: str
        if not delimiter:
            delimiter = None
        stop_pattern = dimension['stop_pattern'] # type: str

        stop = False

        while stop == False:
            table_values = self.static_values.copy()
            line = in_fh.readline()
            #pdb.set_trace()
            stop_match = re.match(stop_pattern, line)
            if stop_match:
                stop = True
            else:
                elements = line.split(delimiter)
                #self.logger.info(elements)
                table_values['dimension'] = dimension['name']
                table_values['index'] = elements[index_element]
                table_values['value'] = elements[value_element]

                # Convert all values to column ordered strings
                strings = [str(table_values[column]) for column in self.columns]
                out_str = ','.join(strings)
                #self.out_fh.write(out_str)
                print(out_str)

    def _parse_match(self, match, in_fh):
        """Parse single line pattern match.
        """
        dimension_name = list(match.groupdict().keys())[0]
        row_type = self.dimensions[dimension_name]['row_type']
        if row_type == 'single':
            value = list(match.groupdict().values())[0]
            self._write_single_row(dimension_name, value)
        elif row_type == 'series':
            dimension = self.dimensions[dimension_name]
            self._write_series_rows(dimension, in_fh)
        else:
            self.logger.error('Invalid dimension type: {}'.format(dimension_type))
            pdb.set_trace()

    def configure(self, config_file):
        """Get parsing schema from JSON file.
        """

        self.logger.info("Configuring FlexTableParser using JSON file: {}".format(config_file))
        with open(config_file, 'r') as config_fh:
            self.config = json.load(config_fh)
        self.columns = self.config['columns']
        self.dimensions = self.config['dimensions']
        self.overlap = bool(self.config['overlap'])

        self.match_patterns = []
        for dimension_name in self.dimensions:
            dimension = self.dimensions[dimension_name]
            regex_pattern = dimension['regex_pattern']
            self.match_patterns.append(regex_pattern)

    def add_static_value(self, key, value):
        """Specify static table column values.
        """
        
        self.static_values.update({key:value})
        self.columns.append(key)

        self.logger.info('Added static values: {}'.format(self.static_values))
        self.logger.info('Updated column list: {}'.format(self.columns))

    def parse_file(self, in_file):
        """Parse file.
        """
        with open(in_file, 'r') as in_fh:
            self.logger.info('Parsing file: {}'.format(in_file))
            while in_fh:
                line = in_fh.readline()
                if not line:
                    self.logger.info('End of file: {}\n'.format(in_file))
                    break
                # Dev: what does this return
                matches = [re.match(regex, line) for regex in self.match_patterns]
                #true_matches = list(ifilter(lambda x: x > 0, matches))
                true_matches = [x for x in matches if x != None]
                filter 
                if len(true_matches) == 0:
                    continue
                elif len(true_matches) > 1:
                    self.logger.warning('Multiple regex patterns matched this line.')
                    self.logger.warning(line.strip())

                    if self.overlap:
                        # Potential for error if you have overlapping matches 
                        # & and is a series
                        for match in true_matches:
                            self._parse_match(match, in_fh)
                    else:
                        self.logger.error('Error: Configuration file does not allow for overlapping patterns')
                        sys.exit()

                elif len(true_matches) == 1:
                    self.logger.info('Found single match for line: ')
                    self.logger.info(line.strip())
                    self._parse_match(true_matches[0], in_fh)
                        


#!/usr/bin/env python
"""Command-line tool for converting text2table using FlexTableParser.
"""

__author__ = 'pbilling@stanford.edu (Paul Billing-Ross)'

import sys
import argparse
import unittest
from pkg_resources import resource_filename

from . import flextableparser

RSC_PACKAGE = __name__
BUILT_IN_SCHEMAS = {
                    'fastqc': resource_filename(
                                                RSC_PACKAGE, 
                                                'table_schemas/fastqc.json'),
                    'flagstat': resource_filename(
                                                  RSC_PACKAGE, 
                                                  'table_schemas/flagstat.json'),
                    'vcfstats': resource_filename(
                                                  RSC_PACKAGE,
                                                  'table_schemas/vcfstats.json'),
                    'vcfhethom': resource_filename(
                                                   RSC_PACKAGE,
                                                   'table_schemas/vcfhethom.json')
                   }

class Text2Table():

    def __init__(self, argv):

        # Create table parser
        table_parser = flextableparser.FlexTableParser()

        # Parse command-line arguments
        #print(sys.argv)
        args = self.parse_args(argv[1:])

        # Configure parser with built-in schema or specify custom config file
        if args.schema:
            schema_file = BUILT_IN_SCHEMAS[args.schema]
        elif args.json_file:
            schema_file = json_file
        table_parser.configure(schema_file)

        if args.static_values:
            dict_items = (item.split('=') for item in args.static_values.split(','))
            for item in dict_items:
                print(item)
                table_parser.add_static_value(item[0], item[1])

        if args.output_file:
            sys.stdout = open(args.output_file, 'w')

        # Convert input files to table format
        for input_file in args.input_files:  
            table_parser.parse_file(input_file)

    def parse_args(self, argv):

        parser = argparse.ArgumentParser()
        parser.add_argument(
                            "input_files",
                            type = str,
                            nargs = '+',
                            help = 'Input file path')
        parser.add_argument(
                            "-s",
                            "--schema",
                            dest = "schema",
                            type = str,
                            help = "Use built-in table conversion schema",
                            choices = ["fastqc", "flagstat", "vcfstats", "vcfhethom"],
                            required = False)
        parser.add_argument(
                            "-j",
                            "--json-file",
                            dest = "json_file",
                            type = str,
                            help = "Use custom JSON config file with table conversion schema. Only required if not using '-s' option.",
                            required = False)
        parser.add_argument(
                            "-v",
                            "--static-values",
                            dest = "static_values",
                            #nargs = '+',
                            help = "Static values to be added as columns. i.e. '-v=series=test,sample=A'",
                            required = False)
        parser.add_argument(
                            "-o",
                            "--output-file",
                            dest = "output_file",
                            type = str,
                            help = "Output file path",
                            required = False)
        parser.add_argument(
                            "-a",
                            "--append",
                            dest = "append",
                            type = bool,
                            help = "Append instead of write output file",
                            default = False)

        if len(argv) < 1:
            print("No arguments specified")
            parser.print_help()
            sys.exit()
        args = parser.parse_args(argv)
        return(args)

def main():
    Text2Table(sys.argv)

if __name__ == '__main__':
    main()




#!/usr/bin/env python

import unittest

from sliders import text2table

class TestText2Table(unittest.TestCase):
    
    def test_single_input(self):
        inputs=[
                './text2table.py',
                '--schema=fastqc', 
                '../tests/sample_fastqc.data']
        text2table.Text2Table(inputs)

    def test_single_input_out(self):
        inputs=[
                './text2table.py',
                '--schema=fastqc',
                '--output-file=../tests/test_fastqc.csv', 
                '../tests/sample_fastqc.data']
        text2table.Text2Table(inputs)

    def test_multiple_input_files(self):
        inputs=[
                './text2table.py', 
                '--schema=fastqc',
                '../tests/sample_fastqc.data',
                '../tests/sample_fastqc_2.data']
        text2table.Text2Table(inputs)

    def test_multiple_input_files_static(self):
        inputs=[
                './text2table.py', 
                '--schema=fastqc',
                '--static-values=series=test,sample=A',
                '../tests/sample_fastqc.data',
                '../tests/sample_fastqc_2.data']
        text2table.Text2Table(inputs)

    def test_static_out(self):
        inputs=[
                './text2table.py',
                '--schema=fastqc',
                '--static-values=series=test,sample=A',
                '--output-file=../tests/test_static_fastqc.csv', 
                '../tests/sample_fastqc.data']
        text2table.Text2Table(inputs)

if __name__ == '__main__':
    unittest.main()